@@ -7,7 +7,11 @@
  * directory or online at http://www.openafs.org/dl/license10.html
  */
 
+#include <afsconfig.h>
 #include <afs/param.h>
+
+RCSID("$Header: /cvs/openafs/src/auth/cellconfig.c,v 1.14 2001/08/08 00:03:37 shadow Exp $");
+
 #include <afs/stds.h>
 #include <afs/pthread_glock.h>
 #ifdef UKERNEL
@@ -26,7 +30,11 @@
 #include <netdb.h>
 #include <sys/file.h>
 #include <sys/time.h>
-#endif
+#ifdef AFS_AFSDB_ENV
+#include <arpa/nameser.h>
+#include <resolv.h>
+#endif /* AFS_AFSDB_ENV */
+#endif /* AFS_NT40_ENV */
 #include <errno.h>
 #include <ctype.h>
 #include <time.h>
@@ -73,6 +81,7 @@ register char *aname; {
     /* lookup a service name */
     struct servent *ts;
     register struct afsconf_servPair *tsp;
+
 #if     defined(AFS_OSF_ENV) || defined(AFS_DEC_ENV)
     ts = getservbyname(aname, "");
 #else
@@ -82,6 +91,7 @@ register char *aname; {
 	/* we found it in /etc/services, so we use this value */
 	return ts->s_port;  /* already in network byte order */
     }
+
     /* not found in /etc/services, see if it is one of ours */
     for(tsp = serviceTable;; tsp++) {
 	if (tsp->name == (char *) 0) return -1;
@@ -214,7 +224,7 @@ register char *adir; {
     LOCK_GLOBAL_MUTEX
     /* zero structure and fill in name; rest is done by internal routine */
     tdir = (struct afsconf_dir *) malloc(sizeof(struct afsconf_dir));
-    bzero(tdir, sizeof(struct afsconf_dir));
+    memset(tdir, 0, sizeof(struct afsconf_dir));
     tdir->name = (char *) malloc(strlen(adir)+1);
     strcpy(tdir->name, adir);
 
@@ -391,7 +401,7 @@ char clones[];
 		curEntry = 0;
 	    }
 	    curEntry = (struct afsconf_entry *) malloc(sizeof(struct afsconf_entry));
-	    bzero(curEntry, sizeof(struct afsconf_entry));
+	    memset(curEntry, 0, sizeof(struct afsconf_entry));
 	    code = ParseCellLine(tbuffer, curEntry->cellInfo.name, linkedcell);
 	    if (code) {
 		afsconf_CloseInternal(adir);
@@ -540,6 +550,100 @@ afsconf_GetExtendedCellInfo(adir, acellName, aservice, acellInfo, clones)
     return code;
 }
 
+#ifdef AFS_AFSDB_ENV
+afsconf_GetAfsdbInfo(acellName, aservice, acellInfo)
+    char *acellName;
+    char *aservice;
+    struct afsconf_cell *acellInfo;
+{
+    afs_int32 code;
+    int tservice, len, i;
+    unsigned char answer[1024];
+    unsigned char *p;
+    char host[256];
+    int server_num = 0;
+    int minttl = 0;
+
+    /* The resolver isn't always MT-safe.. Perhaps this ought to be
+     * replaced with a more fine-grained lock just for the resolver
+     * operations.
+     */
+    LOCK_GLOBAL_MUTEX
+    len = res_search(acellName, C_IN, T_AFSDB, answer, sizeof(answer));
+    UNLOCK_GLOBAL_MUTEX
+
+    if (len < 0)
+	return AFSCONF_NOTFOUND;
+
+    p = answer + sizeof(HEADER);	/* Skip header */
+    code = dn_expand(answer, answer + len, p, host, sizeof(host));
+    if (code < 0)
+	return AFSCONF_NOTFOUND;
+    strncpy(acellInfo->name, host, sizeof(acellInfo->name));
+
+    p += code + QFIXEDSZ;	/* Skip name */
+
+    while (p < answer + len) {
+	int type, ttl, size;
+
+	code = dn_expand(answer, answer + len, p, host, sizeof(host));
+	if (code < 0)
+	    return AFSCONF_NOTFOUND;
+
+	p += code;	/* Skip the name */
+	type = (p[0] << 8) | p[1];
+	p += 4;		/* Skip type and class */
+	ttl = (p[0] << 24) | (p[1] << 16) | (p[2] << 8) | p[3];
+	p += 4;		/* Skip the TTL */
+	size = (p[0] << 8) | p[1];
+	p += 2;		/* Skip the size */
+
+	if (type == T_AFSDB) {
+	    struct hostent *he;
+	    short afsdb_type;
+
+	    afsdb_type = (p[0] << 8) | p[1];
+	    code = dn_expand(answer, answer+len, p+2, host, sizeof(host));
+	    if (code < 0)
+		return AFSCONF_NOTFOUND;
+
+	    if ((afsdb_type == 1) &&
+		(server_num < MAXHOSTSPERCELL) &&
+		/* Do we want to get TTL data for the A record as well? */
+		(he = gethostbyname(host))) {
+		afs_int32 ipaddr;
+		memcpy(&ipaddr, he->h_addr, he->h_length);
+		acellInfo->hostAddr[server_num].sin_addr.s_addr = ipaddr;
+		strncpy(acellInfo->hostName[server_num], host,
+			sizeof(acellInfo->hostName[server_num]));
+		server_num++;
+
+		if (!minttl || ttl < minttl) minttl = ttl;
+	    }
+	}
+
+	p += size;
+    }
+
+    if (server_num == 0)		/* No AFSDB records */
+	return AFSCONF_NOTFOUND;
+    acellInfo->numServers = server_num;
+
+    if (aservice) {
+	tservice = afsconf_FindService(aservice);
+	if (tservice < 0)
+	    return AFSCONF_NOTFOUND;  /* service not found */
+	for (i=0; i<acellInfo->numServers; i++) {
+	    acellInfo->hostAddr[i].sin_port = tservice;
+	}
+    }
+
+    acellInfo->timeout = minttl ? (time(0) + minttl) : 0;
+
+    return 0;
+}
+#endif /* AFS_AFSDB_ENV */
+
 afsconf_GetCellInfo(adir, acellName, aservice, acellInfo)
 struct afsconf_dir *adir;
 char *aservice;
@@ -601,12 +705,17 @@ struct afsconf_cell *acellInfo; {
 		acellInfo->hostAddr[i].sin_port = tservice;
 	    }
 	}
+	acellInfo->timeout = 0;
 	UNLOCK_GLOBAL_MUTEX
 	return 0;
     }
     else {
 	UNLOCK_GLOBAL_MUTEX
+#ifdef AFS_AFSDB_ENV
+	return afsconf_GetAfsdbInfo(acellName, aservice, acellInfo);
+#else
 	return AFSCONF_NOTFOUND;
+#endif /* AFS_AFSDB_ENV */
     }
 }
 
@@ -672,7 +781,7 @@ register struct afsconf_dir *adir; {
     if (adir->keystr) free(adir->keystr);
 
     /* reinit */
-    bzero(adir, sizeof(struct afsconf_dir));
+    memset(adir, 0, sizeof(struct afsconf_dir));
     adir->name = tname;	    /* restore it */
     return 0;
 }
@@ -743,9 +852,13 @@ afsconf_GetKeys(adir, astr)
 struct afsconf_dir *adir;
 struct afsconf_keys *astr;
 {
+    register afs_int32 code;
+
     LOCK_GLOBAL_MUTEX
-    afsconf_Check(adir);
-    bcopy(adir->keystr, astr, sizeof(struct afsconf_keys));
+    code = afsconf_Check(adir);
+    if (code)
+	return AFSCONF_FAILURE;
+    memcpy(astr, adir->keystr, sizeof(struct afsconf_keys));
     UNLOCK_GLOBAL_MUTEX
     return 0;
 }
@@ -761,9 +874,12 @@ afs_int32 afsconf_GetLatestKey(adir, avno, akey)
     register struct afsconf_key *tk;
     register afs_int32 best;
     struct afsconf_key *bestk;
+    register afs_int32 code;
     
     LOCK_GLOBAL_MUTEX
-    afsconf_Check(adir);
+    code = afsconf_Check(adir);
+    if (code)
+	return AFSCONF_FAILURE;
     maxa = adir->keystr->nkeys;
 
     best = -1;	    /* highest kvno we've seen yet */
@@ -776,7 +892,7 @@ afs_int32 afsconf_GetLatestKey(adir, avno, akey)
 	}
     }
     if (bestk) {    /* found any  */
-	if (akey) bcopy(bestk->key, akey, 8); /* copy out latest key */
+	if (akey) memcpy(akey, bestk->key, 8); /* copy out latest key */
 	if (avno) *avno = bestk->kvno;	/* and kvno to caller */
 	UNLOCK_GLOBAL_MUTEX
 	return 0;
@@ -793,14 +909,17 @@ char *akey;
 {
     register int i, maxa;
     register struct afsconf_key *tk;
+    register afs_int32 code;
 
     LOCK_GLOBAL_MUTEX
-    afsconf_Check(adir);
+    code = afsconf_Check(adir);
+    if (code)
+	return AFSCONF_FAILURE;
     maxa = adir->keystr->nkeys;
 
     for(tk = adir->keystr->key,i=0;i<maxa;i++,tk++) {
 	if (tk->kvno == avno) {
-	    bcopy(tk->key, akey, 8);
+	    memcpy(akey, tk->key, 8);
 	    UNLOCK_GLOBAL_MUTEX
 	    return 0;
 	}
@@ -819,7 +938,7 @@ struct afsconf_dir *adir;
     register afs_int32 i;
     char tbuffer[256];
 
-    bcopy(adir->keystr, &tkeys, sizeof(struct afsconf_keys));
+    memcpy(&tkeys, adir->keystr, sizeof(struct afsconf_keys));
 
     /* convert it to net byte order */
     for(i = 0; i<tkeys.nkeys; i++ )
@@ -877,7 +996,7 @@ char akey[8];
 	tkey = &tk->key[tk->nkeys++];
     }
     tkey->kvno = akvno;
-    bcopy(akey, tkey->key, 8);
+    memcpy(tkey->key, akey, 8);
     i = SaveKeys(adir);
     afsconf_Touch(adir);
     UNLOCK_GLOBAL_MUTEX
@@ -913,7 +1032,7 @@ afs_int32 akvno;
     /* otherwise slide the others down.  i and tkey point at the guy to delete */
     for(;i<tk->nkeys-1; i++,tkey++) {
 	tkey->kvno = (tkey+1)->kvno;
-	bcopy((tkey+1)->key, tkey->key, 8);
+	memcpy(tkey->key, (tkey+1)->key, 8);
     }
     tk->nkeys--;
     i = SaveKeys(adir);
